@@ -23,7 +23,6 @@ from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import exceptions
 
 LOG = logging.getLogger(__name__)
-MAX_SLEEP = 8
 
 
 class Transaction(api.Transaction):
@@ -53,10 +52,11 @@ class Transaction(api.Transaction):
         self.ovsdb_connection.queue_txn(self)
         try:
             result = self.results.get(timeout=self.timeout)
-        except queue.Empty:
-            raise exceptions.TimeoutException(commands=self.commands,
-                                              timeout=self.timeout,
-                                              cause='Result queue is empty')
+        except queue.Empty as e:
+            raise exceptions.TimeoutException(
+                commands=self.commands,
+                timeout=self.timeout,
+                cause='Result queue is empty') from e
         if isinstance(result, idlutils.ExceptionResult):
             if self.log_errors:
                 LOG.error(result.tb)
@@ -74,7 +74,6 @@ class Transaction(api.Transaction):
     def do_commit(self):
         self.start_time = time.time()
         attempts = 0
-        retries = 0
         if not self.commands:
             LOG.debug("There are no commands to commit")
             return []
@@ -104,13 +103,7 @@ class Transaction(api.Transaction):
                 # TRY_AGAIN until we time out and Connection.run() calls
                 # idl.run() again. So, call idl.run() here just in case.
                 self.api.idl.run()
-
-                # In the event that there is an issue with the txn or the db
-                # is down, don't spam new txns as fast as we can
-                time.sleep(min(2 ** retries, self.time_remaining(), MAX_SLEEP))
-                retries += 1
                 continue
-            retries = 0
             if status in (txn.ERROR, txn.NOT_LOCKED):
                 msg = 'OVSDB Error: '
                 if status == txn.NOT_LOCKED:
@@ -126,10 +119,10 @@ class Transaction(api.Transaction):
                     # For now, raise similar error to vsctl/utils.execute()
                     raise RuntimeError(msg)
                 return
-            elif status == txn.ABORTED:
+            if status == txn.ABORTED:
                 LOG.debug("Transaction aborted")
                 return
-            elif status == txn.UNCHANGED:
+            if status == txn.UNCHANGED:
                 LOG.debug("Transaction caused no change")
             elif status == txn.SUCCESS:
                 self.post_commit(txn)
